@@ -115,11 +115,16 @@ namespace OrthancPlugins
 
       void AddOrthancJson(const Json::Value& value);
 
-      void AddDicomWebSerializedJson(const void* data,
-                                     size_t size);
+      void AddDicomWebInstanceSerializedJson(const void* data,
+                                             size_t size);
+
+      void AddDicomWebSeriesSerializedJson(const void* data,
+                                           size_t size);
 
       void Send();
 
+      void CloseAndGetJsonOutput(std::string& target);
+
       void AddInstance(const DicomInstance& instance,
                        const std::string& bulkRoot);
     };
