@@ -29,12 +29,18 @@
 #include <HttpServer/HttpContentNegociation.h>
 #include <Logging.h>
 #include <Toolbox.h>
+#include <SerializationToolbox.h>
 #include <MultiThreading/SharedMessageQueue.h>
+#include <Compression/GzipCompressor.h>
 
 #include <memory>
 #include <boost/thread/mutex.hpp>
 #include <boost/thread.hpp>
+#include <boost/lexical_cast.hpp>
+#include <boost/algorithm/string/predicate.hpp>
 
+static const std::string SERIES_METADATA_ATTACHMENT_ID = "4301";
+static std::string WADO_BASE_PLACEHOLDER = "$WADO_BASE_PLACEHOLDER$";
 static const char* const MAIN_DICOM_TAGS = "MainDicomTags";
 static const char* const REQUESTED_TAGS = "RequestedTags";
 static const char* const INSTANCES = "Instances";
@@ -878,7 +884,7 @@ static void WriteInstanceMetadata(OrthancPlugins::DicomWebFormatter::HttpWriter&
   {
     if (buffer.RestApiGet("/instances/" + orthancId + "/attachments/4444/data", false))
     {
-      writer.AddDicomWebSerializedJson(buffer.GetData(), buffer.GetSize());
+      writer.AddDicomWebInstanceSerializedJson(buffer.GetData(), buffer.GetSize());
     }
     else if (buffer.RestApiGet("/instances/" + orthancId + "/file", false))
     {
@@ -894,7 +900,7 @@ static void WriteInstanceMetadata(OrthancPlugins::DicomWebFormatter::HttpWriter&
       }
 
       buffer.RestApiPut("/instances/" + orthancId + "/attachments/4444", dicomweb, false);
-      writer.AddDicomWebSerializedJson(dicomweb.c_str(), dicomweb.size());
+      writer.AddDicomWebInstanceSerializedJson(dicomweb.c_str(), dicomweb.size());
     }
   }
 #endif
@@ -1074,7 +1080,7 @@ void RetrieveDicomInstance(OrthancPluginRestOutput* output,
 }
 
 
-static void GetChildrenIdentifiers(std::list<std::string>& target,
+static void GetChildrenIdentifiers(std::set<std::string>& target,
                                    std::string& resourceDicomUid,
                                    Orthanc::ResourceType level,
                                    const std::string& orthancId)
@@ -1119,7 +1125,7 @@ static void GetChildrenIdentifiers(std::list<std::string>& target,
 
     for (Json::Value::ArrayIndex i = 0; i < children.size(); i++)
     {
-      target.push_back(children[i].asString());
+      target.insert(children[i].asString());
     }
   }  
 }
@@ -1250,7 +1256,7 @@ void InstanceWorkerThread(InstanceWorkerData* data)
           instanceToLoad->bulkRoot = (data->wadoBase +
                               "studies/" + instanceToLoad->studyInstanceUid +
                               "/series/" + instanceToLoad->seriesInstanceUid + 
-                              "/instances/" + instanceResource["MainDicomTags"]["SOPInstanceUID"].asString() + "/bulk");
+                              "/instances/" + instanceResource[MAIN_DICOM_TAGS]["SOPInstanceUID"].asString() + "/bulk");
         }
       }
 
@@ -1278,7 +1284,7 @@ void InstanceWorkerThread(InstanceWorkerData* data)
   }
 }
 
-void RetrieveSeriesMetadataInternal(OrthancPluginRestOutput* output,
+void RetrieveSeriesMetadataInternal(std::set<std::string>& instancesIds,
                                     OrthancPlugins::DicomWebFormatter::HttpWriter& writer,
                                     MainDicomTagsCache& cache,
                                     const OrthancPlugins::MetadataMode& mode,
@@ -1289,7 +1295,6 @@ void RetrieveSeriesMetadataInternal(OrthancPluginRestOutput* output,
                                     const std::string& wadoBase)
 {
   ChildrenMainDicomMaps instancesDicomMaps;
-  std::list<std::string> instancesIds;
   std::string seriesDicomUid;
 
   unsigned int workersCount =  OrthancPlugins::Configuration::GetMetadataWorkerThreadsCount();
@@ -1299,6 +1304,10 @@ void RetrieveSeriesMetadataInternal(OrthancPluginRestOutput* output,
   if (oneLargeQuery)
   {
     GetChildrenMainDicomTags(instancesDicomMaps, seriesDicomUid, Orthanc::ResourceType_Series, seriesOrthancId);
+    for (ChildrenMainDicomMaps::const_iterator it = instancesDicomMaps.begin(); it != instancesDicomMaps.end(); ++it)
+    {
+      instancesIds.insert(it->first);
+    }
   }
   else
   {
@@ -1334,7 +1343,7 @@ void RetrieveSeriesMetadataInternal(OrthancPluginRestOutput* output,
     }
     else
     {
-      for (std::list<std::string>::const_iterator i = instancesIds.begin(); i != instancesIds.end(); ++i)
+      for (std::set<std::string>::const_iterator i = instancesIds.begin(); i != instancesIds.end(); ++i)
       {
         instancesQueue.Enqueue(new InstanceToLoad(*i, "", writerMutex, writer, isXml, OrthancPluginDicomWebBinaryMode_BulkDataUri, studyInstanceUid, seriesInstanceUid));
       }
@@ -1358,18 +1367,190 @@ void RetrieveSeriesMetadataInternal(OrthancPluginRestOutput* output,
   }
   else
   { // old single threaded code
-    std::list<std::string> instances;
+    std::set<std::string> instances;
     std::string seriesDicomUid;  // not used
 
     GetChildrenIdentifiers(instances, seriesDicomUid, Orthanc::ResourceType_Series, seriesOrthancId);
 
-    for (std::list<std::string>::const_iterator i = instances.begin(); i != instances.end(); ++i)
+    for (std::set<std::string>::const_iterator i = instances.begin(); i != instances.end(); ++i)
     {
       WriteInstanceMetadata(writer, mode, cache, *i, studyInstanceUid, seriesInstanceUid, wadoBase);
     }
   }
 }
 
+void CacheSeriesMetadataInternal(std::string& serializedSeriesMetadata,
+                                 OrthancPlugins::DicomWebFormatter::HttpWriter& writer,
+                                 MainDicomTagsCache& cache,
+                                 const std::string& studyInstanceUid, 
+                                 const std::string& seriesInstanceUid, 
+                                 const std::string& seriesOrthancId)
+{
+  Orthanc::GzipCompressor compressor;
+  std::string compressedSeriesMetadata;
+  std::set<std::string> instancesIds;
+
+  // compute the series metadata with a placeholder WADO base url because, the base url might change (e.g if there are 2 Orthanc connected to the same DB)
+  RetrieveSeriesMetadataInternal(instancesIds, writer, cache, OrthancPlugins::MetadataMode_Full, false /* isXml */, seriesOrthancId, studyInstanceUid, seriesInstanceUid, WADO_BASE_PLACEHOLDER);
+  writer.CloseAndGetJsonOutput(serializedSeriesMetadata);
+
+  // save in attachments for future use
+  Orthanc::IBufferCompressor::Compress(compressedSeriesMetadata, compressor, serializedSeriesMetadata);
+  std::string instancesMd5;
+  Orthanc::Toolbox::ComputeMD5(instancesMd5, instancesIds);
+
+  std::string cacheContent = "2;" + instancesMd5 + ";" + compressedSeriesMetadata; 
+
+  Json::Value putResult;
+  std::string attachmentUrl = "/series/" + seriesOrthancId + "/attachments/" + SERIES_METADATA_ATTACHMENT_ID;
+  if (!OrthancPlugins::RestApiPut(putResult, attachmentUrl, cacheContent, false))
+  {
+    LOG(WARNING) << "DicomWEB: failed to write series metadata attachment";
+  }
+
+}
+
+void CacheSeriesMetadata(const std::string& seriesOrthancId)
+{
+  if (!OrthancPlugins::Configuration::IsMetadataCacheEnabled())
+  {
+    return;
+  }
+
+  LOG(INFO) << "DicomWEB: pre-computing the WADO-RS series metadata for series " << seriesOrthancId;
+
+  std::string studyInstanceUid, seriesInstanceUid;
+  
+  Json::Value result;
+  if (OrthancPlugins::RestApiGet(result, "/series/" + seriesOrthancId, false))
+  {
+    seriesInstanceUid = result[MAIN_DICOM_TAGS]["SeriesInstanceUID"].asString();
+    if (OrthancPlugins::RestApiGet(result, "/studies/" + result["ParentStudy"].asString(), false))
+    {
+      studyInstanceUid = result[MAIN_DICOM_TAGS]["StudyInstanceUID"].asString();
+
+      MainDicomTagsCache cache;
+      OrthancPlugins::DicomWebFormatter::HttpWriter writer(NULL /* output */, false /* isXml */);  // we cache only the JSON format -> no need for an HttpOutput
+
+      std::string serializedSeriesMetadataNotUsed;
+      CacheSeriesMetadataInternal(serializedSeriesMetadataNotUsed, writer, cache, studyInstanceUid, seriesInstanceUid, seriesOrthancId);
+    }
+  }
+}
+
+void UpdateSeriesMetadataCache(OrthancPluginRestOutput* output,
+                               const char* /*url*/,
+                               const OrthancPluginHttpRequest* request)
+{
+  if (request->method != OrthancPluginHttpMethod_Post)
+  {
+    OrthancPluginSendMethodNotAllowed(OrthancPlugins::GetGlobalContext(), output, "POST");
+    return;
+  }
+
+  if (request->groupsCount != 1)
+  {
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_BadRequest);
+  }
+
+  if (!OrthancPlugins::Configuration::IsMetadataCacheEnabled())
+  {
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_BadRequest, "The metadata cache is disabled in the Orthanc configuration.");
+  }
+
+  std::string studyId(request->groups[0]);
+
+  LOG(INFO) << "DicomWEB: updating the series metadata cache for study " << studyId;
+
+  Json::Value study;
+
+  if (OrthancPlugins::RestApiGet(study, "/studies/" + studyId, false) && study.type() == Json::objectValue)
+  {
+    for (Json::ArrayIndex i = 0; i < study["Series"].size(); ++i)
+    {
+      CacheSeriesMetadata(study["Series"][i].asString());
+    }
+  }
+
+  std::string answer = "{}"; 
+  OrthancPluginAnswerBuffer(OrthancPlugins::GetGlobalContext(), output, answer.c_str(), answer.size(), "application/json");
+}
+
+
+
+void RetrieveSeriesMetadataInternalWithCache(OrthancPlugins::DicomWebFormatter::HttpWriter& writer,
+                                             MainDicomTagsCache& cache,
+                                             const OrthancPlugins::MetadataMode& mode,
+                                             bool isXml,
+                                             const std::string& seriesOrthancId,
+                                             const std::string& studyInstanceUid,
+                                             const std::string& seriesInstanceUid,
+                                             const std::string& wadoBase)
+{
+  if (OrthancPlugins::Configuration::IsMetadataCacheEnabled() &&
+      mode == OrthancPlugins::MetadataMode_Full && 
+      !isXml)
+  {
+    // check if we already have computed the series metadata and saved them in an attachment
+    std::string serializedSeriesMetadata;
+    std::string cacheContent;
+    bool hasBeenReadFromCache = false;
+    Orthanc::GzipCompressor compressor;
+
+    std::string attachmentUrl = "/series/" + seriesOrthancId + "/attachments/" + SERIES_METADATA_ATTACHMENT_ID;
+
+    if (OrthancPlugins::RestApiGetString(cacheContent, attachmentUrl + "/data", false))
+    {
+      if (boost::starts_with(cacheContent, "2;"))  // version 2, cacheContent is "2;sorted-instances-list-md5;compressedSeriesMetadata"
+      {
+        // check that the instances count have not changed since we have saved the data in cache 
+        // StableSeries event will always overwrite it but this is usefull if retrieving the metadata while
+        // the instances are being received
+        // Note: we can not use Toolbox::SplitString because the compressed metadata contain a lot of ";"
+        const char* secondSemiColon = strchr(&cacheContent[2], ';');
+        std::string instancesMd5InCache(&cacheContent[2], secondSemiColon - &cacheContent[2]);
+        std::string compressedSeriesMetadata(secondSemiColon + 1, cacheContent.size() - (secondSemiColon+1 - cacheContent.c_str()));
+        
+        Json::Value seriesInfo;
+
+        if (!OrthancPlugins::RestApiGet(seriesInfo, "/series/" + seriesOrthancId, false))
+        {
+          throw Orthanc::OrthancException(Orthanc::ErrorCode_UnknownResource);
+        }
+        std::set<std::string> currentInstancesIds;
+        Orthanc::SerializationToolbox::ReadSetOfStrings(currentInstancesIds, seriesInfo, "Instances");
+        std::string currentInstancesMd5;
+        Orthanc::Toolbox::ComputeMD5(currentInstancesMd5, currentInstancesIds);
+
+        if (currentInstancesMd5 == instancesMd5InCache)
+        {
+          Orthanc::IBufferCompressor::Uncompress(serializedSeriesMetadata, compressor, compressedSeriesMetadata);
+
+          hasBeenReadFromCache = true;
+        }
+      }
+    }
+
+    if (!hasBeenReadFromCache)  // regenerate and overwrite current cache
+    {
+      MainDicomTagsCache tmpCache;
+      OrthancPlugins::DicomWebFormatter::HttpWriter tmpWriter(NULL /* output */, false /* isXml */);  // we cache only the JSON format -> no need for an HttpOutput
+
+      CacheSeriesMetadataInternal(serializedSeriesMetadata, tmpWriter, tmpCache, studyInstanceUid, seriesInstanceUid, seriesOrthancId);
+    }
+
+    boost::replace_all(serializedSeriesMetadata, WADO_BASE_PLACEHOLDER, wadoBase);
+
+    writer.AddDicomWebSeriesSerializedJson(serializedSeriesMetadata.c_str(), serializedSeriesMetadata.size());
+  }
+  else
+  {
+    std::set<std::string> instancesIdsNotUsed;
+    RetrieveSeriesMetadataInternal(instancesIdsNotUsed, writer, cache, mode, isXml, seriesOrthancId, studyInstanceUid, seriesInstanceUid, wadoBase);
+  }
+
+}
+
 
 void RetrieveSeriesMetadata(OrthancPluginRestOutput* output,
                             const char* /*url*/,
@@ -1389,7 +1570,7 @@ void RetrieveSeriesMetadata(OrthancPluginRestOutput* output,
   if (LocateSeries(output, seriesOrthancId, studyInstanceUid, seriesInstanceUid, request))
   {
     std::string wadoBase = OrthancPlugins::Configuration::GetBasePublicUrl(request);
-    RetrieveSeriesMetadataInternal(output, writer, cache, mode, isXml, seriesOrthancId, studyInstanceUid, seriesInstanceUid, wadoBase);
+    RetrieveSeriesMetadataInternalWithCache(writer, cache, mode, isXml, seriesOrthancId, studyInstanceUid, seriesInstanceUid, wadoBase);
   }
 
   writer.Send();
@@ -1415,17 +1596,17 @@ void RetrieveStudyMetadata(OrthancPluginRestOutput* output,
 
     std::string wadoBase = OrthancPlugins::Configuration::GetBasePublicUrl(request);
     
-    std::list<std::string> series;
+    std::set<std::string> series;
     std::string studyDicomUid;
     GetChildrenIdentifiers(series, studyDicomUid, Orthanc::ResourceType_Study, studyOrthancId);
 
-    for (std::list<std::string>::const_iterator s = series.begin(); s != series.end(); ++s)
+    for (std::set<std::string>::const_iterator s = series.begin(); s != series.end(); ++s)
     {
-      std::list<std::string> instances;
+      std::set<std::string> instances;
       std::string seriesDicomUid;
       GetChildrenIdentifiers(instances, seriesDicomUid, Orthanc::ResourceType_Series, *s);
 
-      RetrieveSeriesMetadataInternal(output, writer, cache, mode, isXml, *s, studyDicomUid, seriesDicomUid, wadoBase);
+      RetrieveSeriesMetadataInternalWithCache(writer, cache, mode, isXml, *s, studyDicomUid, seriesDicomUid, wadoBase);
     }
 
     writer.Send();
