@@ -31,6 +31,8 @@
 #include <boost/algorithm/string/replace.hpp>
 #include <boost/lexical_cast.hpp>
 
+static bool pluginCanDownloadTranscodedFile = false;
+
 
 static void TokenizeAndNormalize(std::vector<std::string>& tokens,
                                  const std::string& source,
@@ -111,33 +113,24 @@ static bool ParseTransferSyntax(Orthanc::DicomTransferSyntax& syntax,
 
       if (type == "application/octet-stream")
       {
-        if (transferSyntax.empty() ||
-            transferSyntax == "1.2.840.10008.1.2.1")
+        if (transferSyntax.empty())
         {
           syntax = Orthanc::DicomTransferSyntax_LittleEndianExplicit;
           return true;
         }
-        else if (transferSyntax == "1.2.840.10008.1.2")
-        {
-          syntax = Orthanc::DicomTransferSyntax_LittleEndianImplicit;
-          return true;
-        }
-        else if (transferSyntax == "1.2.840.10008.1.2.2")  // New in 1.3
-        {
-          syntax = Orthanc::DicomTransferSyntax_BigEndianExplicit;
-          return false;
-        }        
         else if (transferSyntax == "*")
         {
-          // New in DICOMweb plugin 1.1
-          return false;
+          // don't change transferSyntax, it must have been set to the 'current' value before calling this method
+          return true;
         }
         else
         {
-          throw Orthanc::OrthancException(
-            Orthanc::ErrorCode_BadRequest,
-            "DICOMweb RetrieveFrames: Cannot specify a transfer syntax (" + 
-            transferSyntax + ") for default Little Endian uncompressed pixel data");
+          if (!Orthanc::LookupTransferSyntax(syntax, transferSyntax))
+          {
+            throw Orthanc::OrthancException(Orthanc::ErrorCode_NotImplemented,
+                                            "Unknown transfer syntax in 'Accept' header: " + transferSyntax);
+          }
+          return true;         
         }
       }
       else
@@ -287,7 +280,7 @@ static bool ParseTransferSyntax(Orthanc::DicomTransferSyntax& syntax,
     }
   }
 
-  // By default, DICOMweb expectes Little Endian uncompressed pixel data
+  // By default, DICOMweb expects Little Endian uncompressed pixel data
   syntax = Orthanc::DicomTransferSyntax_LittleEndianExplicit;
   return true;
 }
@@ -435,44 +428,73 @@ static void RetrieveFrames(OrthancPluginRestOutput* output,
                            std::list<unsigned int>& frames)
 {
   std::string orthancId, studyInstanceUid, seriesInstanceUid, sopInstanceUid;
-  OrthancPlugins::MemoryBuffer content;
-  if (LocateInstance(output, orthancId, studyInstanceUid, seriesInstanceUid, sopInstanceUid, request) &&
-      content.RestApiGet("/instances/" + orthancId + "/file", false))
+  if (LocateInstance(output, orthancId, studyInstanceUid, seriesInstanceUid, sopInstanceUid, request))
   {
-    if (allFrames)
-    {
-      OrthancPlugins::LogInfo("DICOMweb RetrieveFrames on " + orthancId + ", all frames");
-    }
-    else
-    {
-      std::string s = "DICOMweb RetrieveFrames on " + orthancId + ", frames: ";
-      for (std::list<unsigned int>::const_iterator 
-             frame = frames.begin(); frame != frames.end(); ++frame)
-      {
-        s += boost::lexical_cast<std::string>(*frame + 1) + " ";
-      }
-
-      OrthancPlugins::LogInfo(s);
-    }
-
-    Orthanc::DicomTransferSyntax targetSyntax;
-
+    OrthancPlugins::MemoryBuffer content;
+    Orthanc::DicomTransferSyntax currentSyntax;
     std::unique_ptr<OrthancPlugins::DicomInstance> instance;
+
+    { // logging only
+      if (allFrames)
+      {
+        OrthancPlugins::LogInfo("DICOMweb RetrieveFrames on " + orthancId + ", all frames");
+      }
+      else
+      {
+        std::string s = "DICOMweb RetrieveFrames on " + orthancId + ", frames: ";
+        for (std::list<unsigned int>::const_iterator 
+              frame = frames.begin(); frame != frames.end(); ++frame)
+        {
+          s += boost::lexical_cast<std::string>(*frame + 1) + " ";
+        }
+
+        OrthancPlugins::LogInfo(s);
+      }
+    }
+
+    std::string currentSyntaxString;
+    if (!OrthancPlugins::RestApiGetString(currentSyntaxString, "/instances/" + orthancId + "/metadata/TransferSyntax", false))
+    {
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError, "DICOMWeb: Unable to get TransferSyntax for instance " + orthancId);
+    }
+
+    if (!Orthanc::LookupTransferSyntax(currentSyntax, currentSyntaxString))
+    {
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_NotImplemented, "Unknown transfer syntax: " + currentSyntaxString);
+    }
+
+    Orthanc::DicomTransferSyntax targetSyntax = currentSyntax;  
+    bool transcodeThisInstance = false;
+    
     if (ParseTransferSyntax(targetSyntax, request))
     {
-      OrthancPlugins::LogInfo("DICOMweb RetrieveFrames: Transcoding instance " + orthancId + 
-                              " to transfer syntax " + Orthanc::GetTransferSyntaxUid(targetSyntax));
-
-      instance.reset(OrthancPlugins::DicomInstance::Transcode(
-                       content.GetData(), content.GetSize(), GetTransferSyntaxUid(targetSyntax)));
+      transcodeThisInstance = targetSyntax != currentSyntax;
     }
-    else
+
+    // maximize the use the Orthanc storage cache.  Since 1.12.2, transcoded file may be stored in the storage cache
+    if (pluginCanDownloadTranscodedFile && transcodeThisInstance)
     {
-      instance.reset(new OrthancPlugins::DicomInstance(content.GetData(), content.GetSize()));
-      if (!Orthanc::LookupTransferSyntax(targetSyntax, instance->GetTransferSyntaxUid()))
+      if (!content.RestApiGet("/instances/" + orthancId + "/file?transcode=" + Orthanc::GetTransferSyntaxUid(targetSyntax), false))
       {
-        throw Orthanc::OrthancException(Orthanc::ErrorCode_NotImplemented,
-                                        "Unknown transfer syntax: " + std::string(GetTransferSyntaxUid(targetSyntax)));
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError, "DICOMWeb: Unable to get transcoded file for for instance " + orthancId);
+      }
+      instance.reset(new OrthancPlugins::DicomInstance(content.GetData(), content.GetSize()));
+    }
+    else // pre 1.12.2 code (or no transcoding needed)
+    {
+      if (!content.RestApiGet("/instances/" + orthancId + "/file", false))
+      {
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError, "DICOMWeb: Unable to get file for for instance " + orthancId);
+      }
+      instance.reset(new OrthancPlugins::DicomInstance(content.GetData(), content.GetSize()));
+
+      if (transcodeThisInstance)
+      {
+        OrthancPlugins::LogInfo("DICOMweb RetrieveFrames: Transcoding instance " + orthancId + 
+                                " to transfer syntax " + Orthanc::GetTransferSyntaxUid(targetSyntax));
+
+        instance.reset(OrthancPlugins::DicomInstance::Transcode(
+                       content.GetData(), content.GetSize(), GetTransferSyntaxUid(targetSyntax)));
       }
     }
 
@@ -513,3 +535,8 @@ void RetrieveSelectedFrames(OrthancPluginRestOutput* output,
   ParseFrameList(frames, request);
   RetrieveFrames(output, request, false, frames);
 }
+
+void SetPluginCanDownloadTranscodedFile(bool enable)
+{
+  pluginCanDownloadTranscodedFile = enable;
+}
\ No newline at end of file
