@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -934,7 +935,7 @@ static void AnswerFrameRendered(OrthancPluginRestOutput* output,
         tags[PHOTOMETRIC_INTERPRETATION].type() == Json::stringValue)
     {
       std::string s = tags[PHOTOMETRIC_INTERPRETATION].asString();
-      Orthanc::Toolbox::StripSpaces(s);
+      s = Orthanc::Toolbox::StripSpaces(s);
       if (s == "MONOCHROME1")
       {
         invert = true;
