@@ -3,7 +3,8 @@
 # Orthanc - A Lightweight, RESTful DICOM Store
 # Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
 # Department, University Hospital of Liege, Belgium
-# Copyright (C) 2017-2020 Osimis S.A., Belgium
+# Copyright (C) 2017-2023 Osimis S.A., Belgium
+# Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
 #
 # This program is free software: you can redistribute it and/or
 # modify it under the terms of the GNU Affero General Public License
@@ -59,8 +60,8 @@ for i in range(2, len(sys.argv)):
 body += bytearray('--%s--' % boundary, 'ascii')
 
 headers = {
-    'Content-Type' : 'multipart/related; type=application/dicom; boundary=%s' % boundary,
-    'Accept' : 'application/json',
+    'Content-Type' : 'multipart/related; type="application/dicom"; boundary=%s' % boundary,
+    'Accept' : 'application/dicom+json',
     }
 
 # Do the HTTP POST request to the STOW-RS server
@@ -73,7 +74,11 @@ else:
     def gen():
         chunkSize = 1024 * 1024
 
-        l = len(body) / chunkSize
+        if sys.version_info[0] == 2:
+            l = len(body) / chunkSize
+        else:
+            l = len(body) // chunkSize
+            
         for i in range(l):
             pos = i * chunkSize
             yield body[pos : pos + chunkSize]
